local GLYPH_POSITIONS = {
	{ "CENTER", -15, 140 },
	{ "CENTER", -14, -103 },
	{ "TOPLEFT", 28, -133 },
	{ "BOTTOMRIGHT", -56, 168 },
	{ "TOPRIGHT", -56, -133 },
	{ "BOTTOMLEFT", 26, 168 },
}
local function GlyphFrame_Update(self)
	for id, position in ipairs(GLYPH_POSITIONS) do
		local glyph = self.glyphs[id] or Talented:MakeGlyph(self, id)
		self.glyphs[id] = glyph
		glyph:ClearAllPoints()
		glyph:SetPoint(unpack(position))
		glyph:Update()
	end
end

local function GlyphFrame_OnEvent(self, event, ...)
	if event == "GLYPH_ADDED" or event == "GLYPH_UPDATED" or event == "GLYPH_REMOVED" then
		local id = ...
		local glyph = self.glyphs[id]
		glyph:Update()
		local _, type = GetGlyphSocketInfo(id, self.group)
		if event == "GLYPH_REMOVED" then
			PlaySound(type == 2 and "Glyph_MinorDestroy" or "Glyph_MajorDestroy")
		else
			PlaySound(type == 2 and "Glyph_MinorCreate" or "Glyph_MajorCreate")
		end
		self.glow:Show()
	else
		self:Update()
	end
end

local function GlyphFrame_OnEnter(self)
	if SpellIsTargeting() then SetCursor"CAST_ERROR_CURSOR" end
end

local function GlyphFrame_OnLeave(self)
	SetCursor()
end

function Talented:MakeGlyphFrame(parent, group)
	local frame = CreateFrame("Frame", nil, parent)
	frame.group = group

	frame:SetWidth(384)
	frame:SetHeight(512)

	local t = frame:CreateTexture(nil, "ARTWORK")
	t:SetTexture"Interface\\Spellbook\\UI-GlyphFrame"
	t:SetWidth(352)
	t:SetHeight(441)
	t:SetPoint"TOPLEFT"
	t:SetTexCoord(0, 0.6875, 0, 0.861328125)

	local fs = frame:CreateFontString(nil, "ARTWORK")
	fs:SetFont"GameFontNormal"
	fs:SetText(GLYPHS)
	fs:SetPoint("TOP", 0, -17)
	frame.title = fs

	local glow = frame:CreateTexture(nil, "OVERLAY")
	glow:Hide()
	glow:SetWidth(352)
	glow:SetHeight(441)
	glow:SetPoint("TOPLEFT", -9, -38)
	glow:SetTexCoord(0, 0.6875, 0, 0.861328125)
	frame.glow = glow

--~ 	local sparkle = frame:CreateTexture(nil, "OVERLAY", "SparkleTextureNormal")
--~ 	sparkle:SetWidth(24)
--~ 	sparkle:SetHeight(24)
--~ 	sparkle:SetPoint("CENTER", -12, 17)
--~ 	frame.sparkle = sparkle

	frame:RegisterEvent"GLYPH_ADDED"
	frame:RegisterEvent"GLYPH_REMOVED"
	frame:RegisterEvent"GLYPH_UPDATED"
	frame:RegisterEvent"GLYPHFRAME_OPEN"
	frame:RegisterEvent"PLAYER_LEVEL_UP"

	frame:SetScript("OnEvent", GlyphFrame_OnEvent)
	frame:SetScript("OnEnter", GlyphFrame_OnEnter)
	frame:SetScript("OnLeave", GlyphFrame_OnLeave)

	frame.Update = GlyphFrame_Update

	frame.glyphs = {}

	frame:Update()
	return frame
end
