local select = select
local ipairs = ipairs
local GetTalentInfo = GetTalentInfo
local GRAY_FONT_COLOR = GRAY_FONT_COLOR
local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR
local GREEN_FONT_COLOR = GREEN_FONT_COLOR
local RED_FONT_COLOR = RED_FONT_COLOR
local LIGHTBLUE_FONT_COLOR = { r = 0.5, g = 0.6, b = 1 }
local GRAY_FONT_COLOR = GRAY_FONT_COLOR
local Talented = Talented

local L = LibStub("AceLocale-3.0"):GetLocale("Talented")

function Talented:UpdateCurrentTemplate()

	local class = select(2, UnitClass"player")
	local info = self:UncompressSpellData(class)
	if not self.alternates then self.alternates = {} end
	for talentGroup = 1, GetNumTalentGroups() do
		local template = self.alternates[talentGroup]
		if not template then
			template = {
				talentGroup = talentGroup,
				name = "Spec"..talentGroup,
				class = class,
			}
		end
		local total = 0
		for tab, tree in ipairs(info) do
			local ttab = template[tab]
			if not ttab then
				ttab = {}
				template[tab] = ttab
			end
			for index = 1, #tree do
				local rank = select(5, GetTalentInfo(tab, index, nil, nil, talentGroup))
				ttab[index] = rank
				total = total + rank
			end
		end
		if talentGroup == GetActiveTalentGroup() then
			self.current = template
			self.maxpoints = total + GetUnspentTalentPoints(nil, nil, talentGroup)
		end
		self.alternates[talentGroup] = template
		if self.template == template then
			self:UpdateTooltip()
		end
	end
end

function Talented:UpdateView()
	if not self.base then return end
	local template, target
	if self.mode == "apply" then
		template = self.current
	else
		template = self.template
	end
	self:UnpackTemplate(template)
	local total = 0

	if template == self.current then
		self:MakeTarget()
		target = self.db.char.target and self.target
	end
	if not template.talentGroup then
		self.base.editname:SetText(template.name)
	end
	if target then
		self.base.targetname:Show()
		self.base.targetname:SetFormattedText(L["Target: %s"], self.db.char.target)
	else
		self.base.targetname:Hide()
	end
	local info = self:UncompressSpellData(template.class)
	local at_cap = self:IsTemplateAtCap(template)
	for tab, tree in ipairs(info) do
		local count = 0
		for index, talent in ipairs(tree) do
			local rank = template[tab][index]
			if not rank then
				template[tab][index] = 0
				rank = 0
			end
			count = count + rank
			local button = self:GetUIElement(tab, index)
			local color = GRAY_FONT_COLOR
			local state = self:GetTalentState(template, tab, index)
			if state == "empty" and (at_cap or self.mode == "view") then
				state = "unavailable"
			end
			if state == "unavailable" then
				button.texture:SetDesaturated(1)
				button.slot:SetVertexColor(0.65, 0.65, 0.65)
				button.rank:Hide()
				button.rank.texture:Hide()
			else
				button.rank:Show()
				button.rank.texture:Show()
				button.rank:SetText(rank)
				button.texture:SetDesaturated(0)
				if state == "full" then
					color = NORMAL_FONT_COLOR
				else
					color = GREEN_FONT_COLOR
				end
				button.slot:SetVertexColor(color.r, color.g, color.b)
				button.rank:SetVertexColor(color.r, color.g, color.b)
			end
			local req = talent.req
			if req then
				local ecolor = color
				if ecolor == GREEN_FONT_COLOR then
					if self.mode == "edit" then
						local s = self:GetTalentState(template, tab, req)
						if s ~= "full" then
							ecolor = RED_FONT_COLOR
						end
					else
						ecolor = NORMAL_FONT_COLOR
					end
				end
				for _, element in ipairs(self:GetUIElement(tab, index, req)) do
					element:SetVertexColor(ecolor.r, ecolor.g, ecolor.b)
				end
			end
			local targetvalue = target and target[tab][index]
			if targetvalue and (targetvalue > 0 or rank > 0) then
				local btarget = self:GetButtonTarget(button)
				btarget:Show()
				btarget.texture:Show()
				btarget:SetText(targetvalue)
				local tcolor
				if rank < targetvalue then
					tcolor = LIGHTBLUE_FONT_COLOR
				elseif rank == targetvalue then
					tcolor = GRAY_FONT_COLOR
				else
					tcolor = RED_FONT_COLOR
				end
				btarget:SetVertexColor(tcolor.r, tcolor.g, tcolor.b)
			elseif button.target then
				button.target:Hide()
				button.target.texture:Hide()
			end
		end
		local frame = self:GetUIElement(tab)
		frame.name:SetFormattedText(L["%s (%d)"], self.tabdata[template.class][tab].name, count)
		total = total + count
		local clear = frame.clear
		if self.mode ~= "edit" or count <= 0 or self.template == self.current then
			clear:Hide()
		else
			clear:Show()
		end
	end
	local points = self.base.points
	local maxpoints = self.maxpoints
	if self.db.profile.show_level_req then
		local level_req = (total == 0) and 1  or (9 + total)
		points:SetFormattedText(L["Level %d"], level_req)
	else
		points:SetFormattedText(L["%d/%d"], total, maxpoints)
	end
	local color
	if total < maxpoints then
		color = GREEN_FONT_COLOR
	elseif total > maxpoints then
		color = RED_FONT_COLOR
	else
		color = NORMAL_FONT_COLOR
	end
	points:SetTextColor(color.r, color.g, color.b)
	if maxpoints ~= total and template.talentGroup then
		self.base.pointsleft:Show()
		self.base.pointsleft.text:SetFormattedText(L["You have %d talent points left"], maxpoints - total)
	else
		self.base.pointsleft:Hide()
	end
end
